/**
 * 读取值的函数
 * @public
 */
type Getter<T> = () => T;
/**
 * 写入值的函数
 * @public
 */
type Setter<T> = (value: T, patch?: boolean) => void;
/**
 * 访问器对象
 * @public
 */
type Accessor<T> = {
    get: Getter<T>;
    set: Setter<T>;
};
/**
 * 信号函数
 * @public
 */
type Signal<T> = (() => T) & ((value: T) => T) & (T extends object ? (value: Partial<T>, patch: boolean) => T : unknown);
/**
 * 响应式对象
 * @public
 */
type Reactive<T> = Signal<T> & (T extends object ? {
    readonly [key in keyof T]: Reactive<T[key]>;
} & (T extends Array<any> ? Array<unknown> : {}) : {});
/**
 * 只读响应式对象
 * @public
 */
type ReadonlyReactive<T> = Getter<T> & (T extends object ? {
    readonly [key in keyof T]: ReadonlyReactive<T[key]>;
} : {});
/**
 * 依赖函数的类型
 * @public
 */
type EffectFunction<T extends any[] = [], R = void> = (...args: T) => R;
/**
 * 响应式对象的内部值
 * @public
 */
type ReactiveType<T> = T extends Reactive<infer V> ? V : never;
/**
 * 依赖函数的依赖项类型
 * @public
 */
type DependenciesType<T> = T extends readonly [infer F, ...infer N] ? [ReactiveType<F>, ...DependenciesType<Readonly<N>>] : T extends Reactive<infer G>[] ? G[] : [];
/**
 * class store 的 getter 属性
 * @public
 */
type Getters<T = Record<string | symbol, any>> = {
    [key in keyof T]: Getter<T[key]>;
};
/**
 * class store 的 action 属性，函数的 this 指向 store
 * @public
 */
type Actions = {
    [key: string]: (...args: any[]) => void;
};
/**
 * class store 的 option 选项
 * @public
 * @param Id - option 的 id
 * @param S - option 的 state
 * @param G - option 的 getters
 * @param A - option 的 actions
 * @returns option
 */
type Options<Id extends string, S extends Record<string | symbol, any>, G extends Getters, A extends Actions> = {
    id: Id;
    state: S;
    getters: G & ThisType<Store<Id, S, G, Record<string, any>>>;
    actions: A & ThisType<Store<Id, S, G, A>>;
};
/**
 * store 的计算属性
 * @public
 */
type Computes<T extends Getters<Record<string | symbol, any>>> = {
    readonly [key in keyof T]: () => ReturnType<T[key]>;
};
/**
 * store 的类型
 * @public
 * @param Id - option 的 id
 * @param S - option 的 state
 * @param G - option 的 getters
 * @param A - option 的 actions
 * @returns store
 */
type Store<Id extends string, S extends Record<string | symbol, any>, G extends Getters, A extends Actions> = {
    $id: Id;
} & Reactive<S> & Computes<G> & Readonly<A>;

declare function createSignal<T>(value: T, path: string): Signal<T>;

/**
 * 使用响应式对象
 * @public
 * @param data - 响应式对象的初始值
 * @returns 响应式对象
 */
declare function useReactive<T>(data: T): Reactive<T>;

/**
 * 清空依赖缓存
 */
declare function clear(): void;
/**
 * 跟踪依赖
 * @param effects - 依赖集合
 * @returns 是否跟踪成功
 */
declare function track(effects: Set<EffectFunction>): boolean;
/**
 * 触发依赖
 * @param effects - 依赖集合
 */
declare function trigger(effects: Set<EffectFunction>): void;
/**
 * 收集依赖函数
 * @public
 * @param effect - 依赖函数
 * @param args - 依赖函数的参数
 * @returns - 依赖函数的返回值
 */
declare function useEffect<T extends [], R>(effect: EffectFunction<T, R>, ...args: T): R;

/**
 * 使用计算属性
 * @public
 * @param getter - 计算属性的 getter 函数
 * @param accessor - 计算属性的 accessor 访问器对象
 * @returns 计算属性
 */
declare function useComputed<T>(getter: Getter<T>): ReadonlyReactive<T>;
declare function useComputed<T>(accessor: Accessor<T>): Reactive<T>;

/**
 * 使用记忆化函数
 * @public
 * @param getter - 记忆化函数的 getter 函数
 * @returns 记忆化函数
 */
declare function useMemo<T>(getter: Getter<T>): Getter<T>;

/**
 * 监听依赖项的变化
 * @public
 * @param cb - 回调函数
 * @param deps - 依赖项
 */
declare function watch<T extends readonly Reactive<any>[]>(cb: (values: DependenciesType<T>, oldVal: DependenciesType<T>) => void, deps: T): void;

/**
 * class类型的store
 * @public
 * @param options - store的选项
 * @returns store
 */
declare function defineStore<Id extends string, S extends Record<string | symbol, any>, G extends Getters, A extends Actions>(options: Options<Id, S, G, A>): Store<Id, S, G, A>;
/**
 * setup类型的store
 * @public
 * @param setup - store的选项
 * @returns store
 */
declare function defineStore<Id extends string, S extends Record<string | symbol, any>, G extends Getters, A extends Actions, Context extends Store<Id, S, G, A>>(setup: (context: Context) => Options<Id, S, G, A>): Store<Id, S, G, A>;
/**
 * class类型的store
 * @param option - store的选项
 * @param ctx - store的上下文
 * @returns store
 */
declare function defineOptionsStore<Id extends string, S extends Record<string | symbol, any>, G extends Getters, A extends Actions>(option: Options<Id, S, G, A>, ctx?: Store<Id, S, G, A>): Store<Id, S, G, A>;
/**
 * setup类型的store
 * @param setup - store的选项
 * @returns store
 */
declare function defineSetupStore<Id extends string, S extends Record<string | symbol, any>, G extends Getters, A extends Actions, Context extends Store<Id, S, G, A>>(setup: (context: Context) => Options<Id, S, G, A>): Store<Id, S, G, A>;

export { Accessor, Actions, Computes, DependenciesType, EffectFunction, Getter, Getters, Options, Reactive, ReactiveType, ReadonlyReactive, Setter, Signal, Store, clear, createSignal, defineOptionsStore, defineSetupStore, defineStore, track, trigger, useComputed, useEffect, useMemo, useReactive, watch };
